@@ -179,7 +179,7 @@ public class AsyncExecutor implements ICounter {
 
             try (final InputStream inputStm = (InputStream) jobProperties.get(Properties.PROP_INPUT_STREAM)) {
                 if (StringUtils.isNotEmpty(m_jobType) && ((null != inputFile) || (null != inputStm))) {
-                    final File tempInputFile = DocumentConverterManager.createTempFile("oxasync", null);
+                    final File tempInputFile = m_manager.createTempFile("oxasync");
 
                     if (null != tempInputFile) {
                         if (null != inputFile) {
@@ -456,7 +456,7 @@ public class AsyncExecutor implements ICounter {
                     m_requestExecutor.execute(asyncRunnable);
 
                 } else {
-                    logDebug("Documentconverter already scheduled same asynchronous job => omitting request", asyncRunnable.getJobProperties());
+                    logDebug("DC already scheduled same asynchronous job => omitting request", asyncRunnable.getJobProperties());
 
                     m_omitCounter.incrementAndGet();
                     asyncRunnable.clear();
