@@ -52,6 +52,7 @@ package com.openexchange.documentconverter;
 
 import java.io.EOFException;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
@@ -61,12 +62,16 @@ import java.net.URL;
 import java.net.URLConnection;
 import java.net.URLDecoder;
 import java.util.HashMap;
-import java.util.UUID;
+import java.util.zip.CRC32;
+import java.util.zip.CheckedInputStream;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.IOUtils;
+import org.apache.commons.io.output.NullOutputStream;
+import org.apache.commons.lang.ArrayUtils;
 import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import com.google.common.base.Throwables;
 
 
 /**
@@ -94,8 +99,10 @@ public class DocumentConverterManager {
     /**
      * Initializes a new {@link DocumentConverterManager}.
      */
-    public DocumentConverterManager() {
+    public DocumentConverterManager(final File tmpRootDir) {
         super();
+
+        m_tmpRootDir = (null != tmpRootDir) ? tmpRootDir : FileUtils.getTempDirectory();
     }
 
     /**
@@ -106,139 +113,54 @@ public class DocumentConverterManager {
     }
 
     /**
-     * @param file
-     * @param locale
-     * @return
+     * @param tmpFilePrefix The mandatory tmp file prefix
+     * @return The tmp file, that was just created of <code>null</code> ij case of an error
      */
-    public static StringBuilder getFileHashBuilder(File file, String locale) {
-        StringBuilder hashBuilder = null;
+    public File createTempFile(@NonNull String tmpFilePrefix) {
+        File ret = null;
 
-        if ((null != file) && file.canRead()) {
-            hashBuilder = new StringBuilder();
+        if ((m_tmpRootDir.isDirectory() && m_tmpRootDir.canWrite()) || mkdirs(m_tmpRootDir)) {
+            // create a temp. file with the name 'prefix*.tmp' within the configured documentconverter cache directory
 
             try {
-                hashBuilder.append(Long.toString(file.length())).append('_').
-                    append(Long.toString(FileUtils.checksumCRC32(file))).append('-');
-            } catch (final Exception e) {
-                logExcp(e);
-            }
-
-            // 'en*' is considered to be the default locale; add all other
-            // locales to the
-            // input file hash in order to be able to perform locale based
-            // conversions
-            if (!StringUtils.isEmpty(locale)) {
-                hashBuilder.append(locale).append('-');
+                ret = File.createTempFile(tmpFilePrefix, ".tmp", m_tmpRootDir);
+            } catch (IOException e) {
+                DocumentConverterManager.logExcp(e);
             }
         }
 
-        return hashBuilder;
+        return ret;
     }
 
     /**
-     * @return
+     * @param tmpDirName The mandatory first directory to create within the tmp root directory
+     * @param tmpDirNameN Optional sub directory names to append to all previews sub directories
+     * @return The tmp directory, that was just created of <code>null</code> in case of an error
      */
-    public static int getHTTPConnectTimeoutMilliseconds(ServerType serverType) {
-        switch (serverType) {
-            case CACHESERVER: {
-                return HTTP_CACHESERVER_CONNECT_TIMEOUT_MS;
-            }
+    public File createTempDir(@NonNull final String tmpDirName, final String... tmpDirNameN) {
+        File ret = createTempFile(tmpDirName);
 
-            case DOCUMENTCONVERTER:
-            default: {
-                return HTTP_DOCUMENTCONVERTER_CONNECT_TIMEOUT_MS;
-            }
-        }
-    }
+        if (ret.canWrite()) {
+            FileUtils.deleteQuietly(ret);
 
-    /**
-     * @return
-     */
-    public static int getHTTPReadTimeoutMilliseconds(ServerType serverType) {
-        switch (serverType) {
-            case CACHESERVER: {
-                return HTTP_CACHESERVER_READ_TIMEOUT_MS;
-            }
-
-            case DOCUMENTCONVERTER:
-            default: {
-                return HTTP_DOCUMENTCONVERTER_READ_TIMEOUT_MS;
-            }
-        }
-    }
-
-    /**
-     * @param prefix
-     * @param fileName
-     * @return
-     */
-    public static File createTempFile(String prefix, String fileName) {
-        return createTempFile(FileUtils.getTempDirectoryPath(), prefix, fileName);
-    }
-
-    /**
-     * @param prefix
-     * @param fileName
-     * @return
-     */
-    public static File createTempFile(@NonNull String tmpRoot, String prefix, String fileName) {
-        File tmpFile = null;
-
-        try {
-            if ((null != fileName) && (fileName.length() > 0)) {
-                final File scratchDir = new File(tmpRoot);
-                final String dirPrefix = ((((null != prefix) && (prefix.length() > 0)) ? prefix : Long.valueOf(System.currentTimeMillis()).toString())) + "_";
-                final long number = System.nanoTime();
-                final long maxIterations = 10000;
-
-                for (long i = number, end = number + maxIterations; i < end; ++i) {
-                    final File tempDir = new File(scratchDir, dirPrefix + i);
-
-                    if (tempDir.mkdirs()) {
-                        tmpFile = new File(tempDir.getAbsolutePath(), fileName);
-                        break;
-                    }
-                }
-            } else {
-                final File scratchDir = new File(tmpRoot);
-
-                if ((scratchDir.isDirectory() && scratchDir.canWrite()) || mkdirs(scratchDir)) {
-                    // create a temp. file with the name 'prefix*.tmp' within the configured documentconverter cache directory
-                    tmpFile = File.createTempFile(prefix, ".tmp", scratchDir);
+            if (ArrayUtils.isNotEmpty(tmpDirNameN)) {
+                for (final String curDirToAdd : tmpDirNameN) {
+                    ret = new File(ret, curDirToAdd);
                 }
             }
-        } catch (final IOException e) {
-            logExcp(e);
-        }
 
-        return tmpFile;
-    }
+            try {
+                FileUtils.forceMkdir(ret);
+            } catch (IOException e) {
+                DocumentConverterManager.logExcp(e);
+            }
 
-    /**
-     * @param tmpRoot
-     * @return
-     */
-    public static File createTempDir(@NonNull String tmpRoot) {
-        final File root = new File(tmpRoot);
-        File tempDirFile = null;
-
-        if (root.canWrite()) {
-            while (null == tempDirFile) {
-                tempDirFile = new File(tmpRoot, UUID.randomUUID().toString());
-
-                try {
-                    FileUtils.forceMkdir(tempDirFile);
-                } catch (IOException e) {
-                    DocumentConverterManager.logExcp(e);
-                }
-
-                if (!tempDirFile.exists()) {
-                    tempDirFile = null;
-                }
+            if (!ret.exists()) {
+                ret = null;
             }
         }
 
-        return tempDirFile;
+        return ret;
     }
 
     /**
@@ -269,7 +191,7 @@ public class DocumentConverterManager {
      * @param filename
      * @return
      */
-    public static byte[] getResourceBuffer(String filename) {
+    public byte[] getResourceBuffer(String filename) {
         ClassLoader loader = Thread.currentThread().getContextClassLoader();
         byte[] resourceBuffer = null;
 
@@ -307,10 +229,10 @@ public class DocumentConverterManager {
      * @param deleteInputFile
      * @return
      */
-    public static File getJobInputFile(@NonNull HashMap<String, Object> jobProperties, @NonNull boolean[] deleteInputFile) {
+    public File getJobInputFile(@NonNull HashMap<String, Object> jobProperties, @NonNull MutableWrapper<Boolean> deleteInputFile) {
         File inputFile = (File) jobProperties.get(Properties.PROP_INPUT_FILE);
 
-        deleteInputFile[0] = false;
+        deleteInputFile.set(Boolean.FALSE);
 
         // if no input file is given, write content of possible InputStream
         // property to a temp file; the conversion is always done using a real
@@ -319,7 +241,7 @@ public class DocumentConverterManager {
         if (null == inputFile) {
             try (final InputStream inputStm = (InputStream) jobProperties.get(Properties.PROP_INPUT_STREAM)) {
                 if (null != inputStm) {
-                    if (null != (inputFile = DocumentConverterManager.createTempFile(FileUtils.getTempDirectoryPath(), "oxcs", null))) {
+                    if (null != (inputFile = createTempFile("oxcs"))) {
                         try {
                             FileUtils.copyInputStreamToFile(inputStm, inputFile);
 
@@ -331,7 +253,7 @@ public class DocumentConverterManager {
 
                             // the temporarily created file needs to be deleted by
                             // the caller of this method later in time
-                            deleteInputFile[0] = true;
+                            deleteInputFile.set(Boolean.TRUE);
                         } catch (final IOException e) {
                             logExcp(e);
 
@@ -362,6 +284,95 @@ public class DocumentConverterManager {
         return inputFile;
     }
 
+    /**
+     * @param file
+     * @param deleteFile
+     * @return
+     */
+    public boolean deleteFileIfFlagged(@NonNull File file, @NonNull MutableWrapper<Boolean> deleteFile) {
+        boolean ret = deleteFile.get().booleanValue();
+
+        if (ret) {
+            FileUtils.deleteQuietly(file);
+            deleteFile.set(Boolean.FALSE);
+        }
+
+        return ret;
+    }
+
+    // static methods
+
+    /**
+     * @param file The source file
+     * @param locale The locale to be used, might be 'null'
+     * @return
+     */
+    public static StringBuilder getFileHashBuilder(File file, String locale) {
+        StringBuilder hashBuilder = null;
+
+        if (null != file) {
+            hashBuilder = new StringBuilder(256);
+
+            try {
+                hashBuilder.append(Long.toString(file.length())).append('_');
+
+                CRC32 checksum = new CRC32();
+
+                try (final InputStream inputStm = new CheckedInputStream(new FileInputStream(file), checksum);
+                     final NullOutputStream outputStm = new NullOutputStream()) {
+
+                    IOUtils.copy(inputStm, outputStm);
+                }
+
+                hashBuilder.append(Long.toString(checksum.getValue())).append('-');
+            } catch (final Exception e) {
+                logExcp(e);
+            }
+
+            // 'en*' is considered to be the default locale; add all other
+            // locales to the
+            // input file hash in order to be able to perform locale based
+            // conversions
+            if (!StringUtils.isEmpty(locale)) {
+                hashBuilder.append(locale).append('-');
+            }
+        }
+
+        return hashBuilder;
+    }
+
+    /**
+     * @return
+     */
+    public static int getHTTPConnectTimeoutMillis(ServerType serverType) {
+        switch (serverType) {
+            case CACHESERVER: {
+                return HTTP_CACHESERVER_CONNECT_TIMEOUT_MS;
+            }
+
+            case DOCUMENTCONVERTER:
+            default: {
+                return HTTP_DOCUMENTCONVERTER_CONNECT_TIMEOUT_MS;
+            }
+        }
+    }
+
+    /**
+     * @return
+     */
+    public static int getHTTPReadTimeoutMillis(ServerType serverType) {
+        switch (serverType) {
+            case CACHESERVER: {
+                return HTTP_CACHESERVER_READ_TIMEOUT_MS;
+            }
+
+            case DOCUMENTCONVERTER:
+            default: {
+                return HTTP_DOCUMENTCONVERTER_READ_TIMEOUT_MS;
+            }
+        }
+    }
+
     // - Logging ---------------------------------------------------------------
 
     /**
@@ -501,25 +512,31 @@ public class DocumentConverterManager {
      */
     static public void logExcp(Exception e) {
         if (isLogError()) {
-            String message = (null != e) ? e.getMessage() : null;
-            final String lowerMessage = (null != message) ? message.toLowerCase() : null;
+            final Throwable rootCause = Throwables.getRootCause(e);
+            final String rootCauseMessage = (null != rootCause) ? rootCause.getMessage() : "";
+            final String lowerCaseRootCauseMessage = rootCauseMessage.toLowerCase();
+            String message = e.getMessage();
+
+            if ((rootCause instanceof SocketException) ||
+                (rootCause instanceof SocketTimeoutException) ||
+                (rootCause instanceof EOFException) ||
+                ((rootCause instanceof IOException) &&
+                 ((lowerCaseRootCauseMessage.contains("broken pipe") || lowerCaseRootCauseMessage.contains(("connection reset")) || lowerCaseRootCauseMessage.contains(("invalid stream")))))) {
 
-            if ((e instanceof SocketException) ||
-                (e instanceof SocketTimeoutException) ||
-                (e instanceof EOFException) ||
-                ((e instanceof IOException) && (null != lowerMessage) && ((lowerMessage.contains("broken pipe") || lowerMessage.contains(("connection reset")) || lowerMessage.contains(("invalid stream")))))) {
                 // ignore but trace socket exceptions; these may occur, when a connection timeout has happened
                 LOG.trace(implGetLogMessage("DC caught an ignorable connection exception: \n" + message, null));
             } else {
-                if (null != message) {
+                if (StringUtils.isNotEmpty(message)) {
                     if (message.length() > Properties.MAX_ERROR_MESSAGE_TOTAL_LENGTH) {
                         final String fillStr = "...";
                         final int beginLength = Properties.MAX_ERROR_MESSAGE_TOTAL_LENGTH - Properties.MAX_ERROR_MESSAGE_END_LENGTH - fillStr.length();
                         final int endLength = Math.min(message.length() - beginLength, Properties.MAX_ERROR_MESSAGE_END_LENGTH);
                         message = message.substring(0, beginLength) + ((endLength > 0) ? ("..." + message.substring(message.length() - endLength)) : "");
                     }
-                } else if (null != e){
-                    message = e.getClass().getName();
+                } else if (StringUtils.isNotEmpty(rootCauseMessage)){
+                    message = rootCauseMessage;
+                } else if (null != rootCause){
+                    message = rootCause.getClass().getName();
                 }
 
                 LOG.error(implGetLogMessage("DC caught an exception: \n" + message, null));
@@ -564,7 +581,9 @@ public class DocumentConverterManager {
 
         if (null != fileName) {
             try {
-                fileName = URLDecoder.decode(fileName, "UTF-8");
+                // #63629: encode all percentage characters being
+                // non-percentage encodings before decoding file name
+                fileName = URLDecoder.decode(fileName.replaceAll("%(?![0-9a-fA-F]{2})", "%25"), "UTF-8");
             } catch (final UnsupportedEncodingException e) {
                 logExcp(e);
             }
@@ -613,4 +632,8 @@ public class DocumentConverterManager {
     // - Static members --------------------------------------------------------
 
     private static Logger LOG = LoggerFactory.getLogger(DocumentConverterManager.class);
+
+    // - Members ---------------------------------------------------------------
+
+    protected File m_tmpRootDir;
 }
