@@ -49,6 +49,9 @@
 
 package com.openexchange.documentconverter;
 
+import java.util.Arrays;
+import java.util.function.Consumer;
+
 //-------------
 //- JobStatus -
 //-------------
@@ -66,7 +69,10 @@ public enum JobError {
     NO_CONTENT(JobError.ERRORCODE_NO_CONTENT),
     MAX_SOURCESIZE(JobError.ERRORCODE_MAX_SOURCESIZE),
     DISPOSED(JobError.ERRORCODE_DISPOSED),
-    UNSUPPORTED(JobError.ERRORCODE_UNSUPPORTED);
+    UNSUPPORTED(JobError.ERRORCODE_UNSUPPORTED),
+    OUT_OF_MEMORY(JobError.ERRORCODE_OUT_OF_MEMORY),
+    ABORT(JobError.ERRORCODE_ABORT),
+    PDFTOOL(JobError.ERRORCODE_PDFTOOL);
 
     /**
      * @param errorCode
@@ -117,6 +123,18 @@ public enum JobError {
                 ret = UNSUPPORTED;
                 break;
 
+            case ERRORCODE_OUT_OF_MEMORY:
+                ret = JobError.OUT_OF_MEMORY;
+                break;
+
+            case ERRORCODE_ABORT:
+                ret = JobError.ABORT;
+                break;
+
+            case ERRORCODE_PDFTOOL:
+                ret = JobError.PDFTOOL;
+                break;
+
             default:
                 ret = GENERAL;
                 break;
@@ -135,6 +153,13 @@ public enum JobError {
                 JobError.NONE;
     }
 
+    /**
+     * @param action
+     */
+    public static void forEach(Consumer<? super JobError> action) {
+       Arrays.stream(values()).forEach(action);
+    }
+
     // - Members ---------------------------------------------------------------
 
     protected int m_errorCode = ERRORCODE_NONE;
@@ -150,4 +175,7 @@ public enum JobError {
     final protected static int ERRORCODE_MAX_SOURCESIZE = 0x00000010;
     final protected static int ERRORCODE_DISPOSED       = 0x00000020;
     final protected static int ERRORCODE_UNSUPPORTED    = 0x00000040;
+    final protected static int ERRORCODE_OUT_OF_MEMORY  = 0x00000080;
+    final protected static int ERRORCODE_ABORT          = 0x00000100;
+    final protected static int ERRORCODE_PDFTOOL     = 0x00000200;
 }
