@@ -78,75 +78,29 @@ public final class PDFExtractor {
 
     final public static int PDFTOOL_ERRORCODE_PASSWORD = 0x00000010;
 
+    /**
+     * Unused
+     */
+    @SuppressWarnings("unused")
+    private PDFExtractor() {
+        // Unused
+    }
+
     /**
      * Initializes a new {@link PDFExtractor}.
      */
-    private PDFExtractor() {
+    public PDFExtractor(@NonNull final DocumentConverterManager manager, @NonNull final File pdfExecutable) throws IOException {
         super();
-    }
 
-    /**
-     * @param pdfExecutable
-     * @param workDir
-     * @return
-     * @throws IOException
-     */
-    public synchronized static PDFExtractor get(
-        @NonNull final File pdfExecutable,
-        @NonNull final File workDir) throws IOException {
-
-
-        if ((null == pdfExtractor) ||
-            !(pdfExecutable.equals(pdfExtractor.implGetPDFExecutable())) ||
-            !(workDir.equals(pdfExtractor.implGetWorkDir()))) {
-
-            try {
-                (pdfExtractor = new PDFExtractor()).init(pdfExecutable, workDir);
-
-                if (LOG.isTraceEnabled()) {
-                    LOG.trace("DC PDF extraction tool found: " + pdfExecutable.getAbsolutePath());
-                }
-            } catch (IOException e) {
-                pdfExtractor = null;
-                throw e;
-            }
-        }
-
-        return pdfExtractor;
-    }
-
-    /**
-     * @param _pdfExecutable
-     * @param _workDir
-     * @throws IOException
-     */
-    protected void init(
-        @NonNull final File _pdfExecutable,
-        @NonNull final File _workDir) throws IOException {
-
-        if (!_pdfExecutable.canExecute()) {
+        if (!pdfExecutable.canExecute()) {
             throw new IOException("DC PDF extraction tool binary cannot be found or executed: " +
-                _pdfExecutable.getAbsolutePath());
+                pdfExecutable.getAbsolutePath());
+        } else if (LOG.isTraceEnabled()) {
+            LOG.trace("DC PDF extraction tool found: " + pdfExecutable.getAbsolutePath());
         }
 
-        boolean workDirValid =
-            (_workDir.exists() && _workDir.canWrite() && _workDir.isDirectory()) ||
-            _workDir.mkdirs();
-
-        if (!workDirValid) {
-            throw new IOException("DC PDF extraction tool working directory is not a directory or not writable: " +
-                _workDir.getAbsolutePath());
-        }
-
-        this.pdfExecutable = _pdfExecutable;
-        this.workDir = _workDir;
-    }
-
-    /**
-     * @return
-     */
-    public File getWorkDir() {
-        return this.workDir;
+        m_manager = manager;
+        m_pdfExecutable = pdfExecutable;
     }
 
     /**
@@ -154,10 +108,10 @@ public final class PDFExtractor {
      * @return
      */
     public int getPageCount(@NonNull final File pdfInputFile) {
-        final File pdfWorkDir = implCreateTempDir();
+        final File pdfWorkDir = m_manager.createTempDir("oxdccpdfpc");
         final File pdfPageCountOutputFile = new File(pdfWorkDir, "pagecount.json");
         final String[] pdfExtractorBinaryCmdLine = {
-            this.pdfExecutable.getAbsolutePath(),
+            this.m_pdfExecutable.getAbsolutePath(),
             "-pagecount",
             pdfInputFile.getAbsolutePath(),
             pdfPageCountOutputFile.getAbsolutePath()
@@ -208,13 +162,18 @@ public final class PDFExtractor {
         int ret = PDFTOOL_ERRORCODE_GENERAL;
 
         final String pages = (pageNumber > 0) ? Integer.toString(pageNumber) : "all";
-        final String resultExtensionSwitch = "png".equalsIgnoreCase(targetFormatShortName) ? "-png" : "-jpg";
         final String width = (pixelWidth > 0) ? Integer.toString(pixelWidth) : "-1";
         final String height = (pixelHeight > 0) ? Integer.toString(pixelHeight) : "-1";
         final String scaling ="cover".equalsIgnoreCase(scaleType) ? "cover" : "contain";
-        final File pdfWorkDir = implCreateTempDir();
-        final String[] pdfExtractorBinaryCmdLine = {
-                this.pdfExecutable.getAbsolutePath(),
+        final File pdfWorkDir = m_manager.createTempDir("oxdccpdf2g");
+        final String resultExtensionSwitch = "png".equalsIgnoreCase(targetFormatShortName) ?
+            "-png" : ("svg".equalsIgnoreCase(targetFormatShortName) ?
+                "-svg" :
+                    "-jpg");
+
+        try {
+            final String[] pdfExtractorBinaryCmdLine = {
+                this.m_pdfExecutable.getAbsolutePath(),
                 resultExtensionSwitch,
                 width,
                 height,
@@ -222,9 +181,8 @@ public final class PDFExtractor {
                 pdfInputFile.getAbsolutePath(),
                 new File(pdfWorkDir, "%d#%d_%dx%d").getAbsolutePath(),
                 pages
-        };
+            };
 
-        try {
             ret = Runtime.getRuntime().exec(pdfExtractorBinaryCmdLine).waitFor();
 
             if (PDFTOOL_ERRORCODE_NONE == ret) {
@@ -242,43 +200,6 @@ public final class PDFExtractor {
         return ret;
     }
 
-    /**
-     * @return
-     */
-    private File implGetPDFExecutable() {
-        return this.pdfExecutable;
-    }
-
-    /**
-     * @return
-     */
-    private File implGetWorkDir() {
-        return this.workDir;
-    }
-
-    /**
-     * @return
-     */
-    private File implCreateTempDir() {
-        File ret = null;
-        int tryNumber = 0;
-
-        while ((null == ret) && (++tryNumber <= 10000)) {
-            final File tmpDir = new File(this.workDir, Long.toString(curDirNumber.incrementAndGet()));
-
-            try {
-                if (!tmpDir.exists()) {
-                    FileUtils.forceMkdir(tmpDir);
-                    ret = tmpDir;
-                }
-            } catch (@SuppressWarnings("unused") IOException e) {
-                // ok. next try
-            }
-        }
-
-        return ret;
-    }
-
     /**
      * @param outputDir The directory, cotaining all extracted page files
      * @param outputFile The file to contain the single extracred page
@@ -345,9 +266,9 @@ public final class PDFExtractor {
 
     // - Members ---------------------------------------------------------------
 
-    private File pdfExecutable = null;
+    private DocumentConverterManager m_manager= null;
 
-    private File workDir = null;
+    private File m_pdfExecutable = null;
 
     // - Static Members --------------------------------------------------------
 
